//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormAbout.h"
#include "FormSettings.h"
#include "FormLog.h"
#include "Log.h"
#include "utils.h"

#include <FileCtrl.hpp>
#include <SysUtils.hpp>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".ini" );
	appSettings.Read(asConfigFile);
	if (this->BorderStyle != bsSingle)
	{
		this->Width = appSettings.frmMain.iWidth;
		this->Height = appSettings.frmMain.iHeight;
	}
	this->Top = appSettings.frmMain.iPosY;
	this->Left = appSettings.frmMain.iPosX;
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.frmMain.bWindowMaximized)
		this->WindowState = wsMaximized;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");

	for (int i=0; i<chboxDirectories->Items->Count; i++)
	{
		if (appSettings.DirsToDelete & ((unsigned int)1 << i))
			chboxDirectories->Checked[i] = true;
	}
	for (int i=0; i<chboxFiles->Items->Count; i++)
	{
		if (appSettings.FilesToDelete & ((unsigned int)1 << i))
			chboxFiles->Checked[i] = true;
	}

	edProjectDir->Text = appSettings.asProjectDir;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".ini" );
	appSettings.frmMain.bWindowMaximized = (this->WindowState == wsMaximized);
	if (!appSettings.frmMain.bWindowMaximized)
	{
		// these values are meaningless is wnd is maximized
		appSettings.frmMain.iWidth = this->Width;
		appSettings.frmMain.iHeight = this->Height;
		appSettings.frmMain.iPosY = this->Top;
		appSettings.frmMain.iPosX = this->Left;
	}
	appSettings.asProjectDir = edProjectDir->Text;	

	appSettings.DirsToDelete = 0;
	appSettings.FilesToDelete = 0;
	for (int i=0; i<chboxDirectories->Items->Count; i++)
	{
		if (chboxDirectories->Checked[i])
			appSettings.DirsToDelete |= ((unsigned int)1 << i);
	}
	for (int i=0; i<chboxFiles->Items->Count; i++)
	{
		if (chboxFiles->Checked[i])
			appSettings.FilesToDelete |= ((unsigned int)1 << i);
	}

	appSettings.Write(asConfigFile);
	
	CanClose = true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::actShowAboutExecute(TObject *Sender)
{
	frmAbout->ShowModal();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowSettingsExecute(TObject *Sender)
{
	frmSettings->appSettings = &appSettings;
	frmSettings->ShowModal();
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	frmLog->SetBufferSize(appSettings.Logging.iMaxUiLogLines);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormShow(TObject *Sender)
{
    static bool once = false;
    if (!once)
    {
		once = true;
		frmLog->SetBufferSize(appSettings.Logging.iMaxUiLogLines);		
		CLog::Instance()->SetLevel(E_LOG_TRACE);
		CLog::Instance()->callbackLog = frmLog->OnLog;
	}
	LOG("Application started");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
	CLog::Instance()->Destroy();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowLogExecute(TObject *Sender)
{
	if (!frmLog->Visible)
		frmLog->Show();
	frmLog->BringToFront();
}
//---------------------------------------------------------------------------

bool MatchPattern(char *element, char *pattern)
{
	if( 0 == StrComp(pattern, "*") )
		return true;

	if( (*element == '\0') && (*pattern != '\0') )
		return false;

	if( *element == '\0' )
		return true;

	switch( *pattern )
	{
        case '*':
            if( MatchPattern(element, &pattern[1]) )
                return true;
            else
                return MatchPattern(&element[1], pattern);

        case '?':
            return MatchPattern(&element[1], &pattern[1]); 
    }

    if( *element == *pattern )
        return MatchPattern(&element[1], &pattern[1]);
    else
        return false;
}

void TfrmMain::CleanDir(AnsiString path)
{
	int iAttributes = faAnyFile;
    Sysutils::TSearchRec sr;
	if (FindFirst(path, iAttributes, sr) == 0)
    {
        do
        {
            // drill into subdirs
            if (sr.Attr & faDirectory) {
                // Do not drill into "." or ".."
                if (sr.Name != "." && sr.Name != "..") {
					for (int i=0; i<chboxDirectories->Items->Count; i++)
					{
						if (chboxDirectories->Checked[i])
						{
							if (chboxDirectories->Items->Strings[i] == sr.Name)
							{
								AnsiString Path = ExtractFilePath(path) + sr.Name;
								SHFILEOPSTRUCT shfo;
								memset(&shfo, 0, sizeof(shfo));
								shfo.hwnd = GetDesktopWindow();
								shfo.wFunc = FO_DELETE;
								// The folder name should not include a trailing backslash but, as with filenames, it should be double null-terminated.
								char *src = new char[Path.Length() + 2];
								memset(src, 0, Path.Length() + 2);
								strcpy(src, Path.c_str());
								shfo.pFrom = src;
								shfo.fFlags = FOF_NOCONFIRMATION | FOF_ALLOWUNDO | FOF_WANTNUKEWARNING ;
								int status = SHFileOperation(&shfo);
								if (!status)
									LOG("Dir %s deleted", src);
								else
									LOG("Failed to delete %s with status %d", src, status);
								delete [] src;
							}
						}
					}

					if(path.Length() + sr.Name.Length() < MAX_PATH-1)
					{
						AnsiString drillPath = appendDirectory(path, sr.Name);
                        CleanDir(drillPath);
                    }
                }
            }
            else
            {
				for (int i=0; i<chboxFiles->Items->Count; i++)
				{
					if (chboxFiles->Checked[i])
					{
						if (MatchPattern(sr.Name.c_str(), chboxFiles->Items->Strings[i].c_str()))
						{
							AnsiString Path = ExtractFilePath(path) + sr.Name;
							bool status = DeleteFile(Path.c_str());
							if (status)
								LOG("File %s deleted", Path.c_str());
							else
								LOG("Failed to delete file %s", Path.c_str());
						}
					}
				}
			}
        } while (FindNext(sr) == 0);
        FindClose(sr);
	}
}


void __fastcall TfrmMain::btnBrowseDirClick(TObject *Sender)
{
    AnsiString asDir;
	if (DirectoryExists(edProjectDir->Text))
		asDir = edProjectDir->Text;
	if (SelectDirectory("Project dir:", "", asDir))
		edProjectDir->Text = asDir;
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::btnCleanClick(TObject *Sender)
{
	AnsiString dir = edProjectDir->Text + "\\*.*";
	CleanDir(dir);
}
//---------------------------------------------------------------------------

